@@ -219,8 +219,8 @@ def _CanPlaceOnSingleLine(uwline):
   indent_amt = style.Get('INDENT_WIDTH') * uwline.depth
   last = uwline.last
   last_index = -1
-  if last.is_comment and re.search(r'^#+\s+pylint:', last.value.strip(),
-                                   re.IGNORECASE):
+  if last.is_comment and re.search(r'^#+\s+pylint:',
+                                   last.value.strip(), re.IGNORECASE):
     last = last.previous_token
     last_index = -2
   if last is None:
@@ -273,6 +273,7 @@ class _StateNode(object):
     return 'StateNode(state=[\n{0}\n], newline={1})'.format(self.state,
                                                             self.newline)
 
+
 # A tuple of (penalty, count) that is used to prioritize the BFS. In case of
 # equal penalties, we prefer states that were inserted first. During state
 # generation, we make sure that we insert states first that break the line as
@@ -281,8 +282,8 @@ _OrderedPenalty = collections.namedtuple('OrderedPenalty', ['penalty', 'count'])
 
 # An item in the prioritized BFS search queue. The 'StateNode's 'state' has
 # the given '_OrderedPenalty'.
-_QueueItem = collections.namedtuple('QueueItem', ['ordered_penalty',
-                                                  'state_node'])
+_QueueItem = collections.namedtuple('QueueItem',
+                                    ['ordered_penalty', 'state_node'])
 
 
 def _AnalyzeSolutionSpace(initial_state):
@@ -466,7 +467,8 @@ def _CalculateNumberOfNewlines(first_token, indent_depth, prev_uwline,
     if not indent_depth:
       # This is a top-level class or function.
       is_inline_comment = prev_last_token.whitespace_prefix.count('\n') == 0
-      if prev_last_token.is_comment and not is_inline_comment:
+      if (not prev_uwline.disable and prev_last_token.is_comment and
+          not is_inline_comment):
         # This token follows a non-inline comment.
         if _NoBlankLinesBeforeCurrentToken(prev_last_token.value, first_token,
                                            prev_last_token):
