@@ -44,6 +44,7 @@ def AssignSubtypes(tree):
   subtype_assigner = _SubtypeAssigner()
   subtype_assigner.Visit(tree)
 
+
 # Map tokens in argument lists to their respective subtype.
 _ARGLIST_TOKEN_TO_SUBTYPE = {
     '=': format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN,
@@ -63,38 +64,38 @@ class _SubtypeAssigner(pytree_visitor.PyTreeVisitor):
     # dictsetmaker ::= (test ':' test (comp_for |
     #                                   (',' test ':' test)* [','])) |
     #                  (test (comp_for | (',' test)* [',']))
+    for child in node.children:
+      self.Visit(child)
+
+    comp_for = False
     dict_maker = False
-    if len(node.children) > 1:
-      index = 0
-      while index < len(node.children):
-        if node.children[index].type != token.COMMENT:
-          break
-        index += 1
-      if index < len(node.children):
-        child = node.children[index + 1]
-        dict_maker = isinstance(child, pytree.Leaf) and child.value == ':'
-    last_was_comma = False
-    last_was_colon = False
+
     for child in node.children:
       if pytree_utils.NodeName(child) == 'comp_for':
+        comp_for = True
         _AppendFirstLeafTokenSubtype(child,
                                      format_token.Subtype.DICT_SET_GENERATOR)
-      else:
+      elif pytree_utils.NodeName(child) == 'COLON':
+        dict_maker = True
+
+    if not comp_for and dict_maker:
+      last_was_colon = False
+      for child in node.children:
         if dict_maker:
-          if last_was_comma:
-            _AppendFirstLeafTokenSubtype(child,
-                                         format_token.Subtype.DICTIONARY_KEY)
-          elif last_was_colon:
-            if pytree_utils.NodeName(child) == 'power':
-              _AppendSubtypeRec(child, format_token.Subtype.NONE)
+          if last_was_colon:
+            if style.Get('INDENT_DICTIONARY_VALUE'):
+              _InsertPseudoParentheses(child)
             else:
               _AppendFirstLeafTokenSubtype(
                   child, format_token.Subtype.DICTIONARY_VALUE)
-            if style.Get('INDENT_DICTIONARY_VALUE'):
-              _InsertPseudoParentheses(child)
-        last_was_comma = isinstance(child, pytree.Leaf) and child.value == ','
-        last_was_colon = isinstance(child, pytree.Leaf) and child.value == ':'
-      self.Visit(child)
+          elif (child is not None and
+                (isinstance(child, pytree.Node) or child.value not in '{:,')):
+            # Mark the first leaf of a key entry as a DICTIONARY_KEY. We
+            # normally want to split before them if the dictionary cannot exist
+            # on a single line.
+            _AppendFirstLeafTokenSubtype(child,
+                                         format_token.Subtype.DICTIONARY_KEY)
+        last_was_colon = pytree_utils.NodeName(child) == 'COLON'
 
   def Visit_expr_stmt(self, node):  # pylint: disable=invalid-name
     # expr_stmt ::= testlist_star_expr (augassign (yield_expr|testlist)
@@ -262,7 +263,10 @@ class _SubtypeAssigner(pytree_visitor.PyTreeVisitor):
     #           | '**' vname)
     #      | vfpdef ['=' test] (',' vfpdef ['=' test])* [','])
     self._ProcessArgLists(node)
-    _SetDefaultOrNamedAssignArgListSubtype(node)
+    for child in node.children:
+      self.Visit(child)
+      if isinstance(child, pytree.Leaf) and child.value == '=':
+        _AppendTokenSubtype(child, format_token.Subtype.VARARGS_LIST)
 
   def Visit_comp_for(self, node):  # pylint: disable=invalid-name
     # comp_for ::= 'for' exprlist 'in' testlist_safe [comp_iter]
@@ -282,8 +286,7 @@ class _SubtypeAssigner(pytree_visitor.PyTreeVisitor):
         _AppendTokenSubtype(
             child,
             subtype=_ARGLIST_TOKEN_TO_SUBTYPE.get(child.value,
-                                                  format_token.Subtype.NONE),
-            force=False)
+                                                  format_token.Subtype.NONE))
 
 
 def _SetDefaultOrNamedAssignArgListSubtype(node):
@@ -307,30 +310,31 @@ def _SetDefaultOrNamedAssignArgListSubtype(node):
             child, format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN_ARG_LIST)
 
 
-def _AppendTokenSubtype(node, subtype, force=True):
+def _AppendTokenSubtype(node, subtype):
   """Append the token's subtype only if it's not already set."""
   pytree_utils.AppendNodeAnnotation(node, pytree_utils.Annotation.SUBTYPE,
                                     subtype)
 
 
-def _AppendFirstLeafTokenSubtype(node, subtype, force=False):
+def _AppendFirstLeafTokenSubtype(node, subtype):
   """Append the first leaf token's subtypes."""
   if isinstance(node, pytree.Leaf):
-    _AppendTokenSubtype(node, subtype, force=force)
+    _AppendTokenSubtype(node, subtype)
     return
-  _AppendFirstLeafTokenSubtype(node.children[0], subtype, force=force)
+  _AppendFirstLeafTokenSubtype(node.children[0], subtype)
 
 
 def _AppendSubtypeRec(node, subtype, force=True):
   """Append the leafs in the node to the given subtype."""
   if isinstance(node, pytree.Leaf):
-    _AppendTokenSubtype(node, subtype, force=force)
+    _AppendTokenSubtype(node, subtype)
     return
   for child in node.children:
     _AppendSubtypeRec(child, subtype, force=force)
 
 
 def _InsertPseudoParentheses(node):
+  """Insert pseudo parentheses so that dicts can be formatted correctly."""
   comment_node = None
   if isinstance(node, pytree.Node):
     if node.children[-1].type == token.COMMENT:
@@ -361,9 +365,12 @@ def _InsertPseudoParentheses(node):
     node.append_child(rparen)
     if comment_node:
       node.append_child(comment_node)
+    _AppendFirstLeafTokenSubtype(node, format_token.Subtype.DICTIONARY_VALUE)
   else:
-    new_node = pytree.Node(syms.atom, [lparen, node.clone(), rparen])
+    clone = node.clone()
+    new_node = pytree.Node(syms.atom, [lparen, clone, rparen])
     node.replace(new_node)
+    _AppendFirstLeafTokenSubtype(clone, format_token.Subtype.DICTIONARY_VALUE)
 
 
 def _GetFirstLeafNode(node):
